#include <WiFi.h>
#include "Adafruit_MQTT.h"
#include "Adafruit_MQTT_Client.h"
#define WLAN_SSID ""
#define WLAN_PASS ""
#define AIO_SERVER "io.adafruit.com"
#define AIO_SERVERPORT 1883  // use 8883 for SSL
#define AIO_USERNAME ""
#define AIO_KEY ""

#include <FastLED.h>

#define LED_PIN 2
#define NUM_LEDS 8
#define BRIGHTNESS 64
#define LED_TYPE WS2811
#define COLOR_ORDER GRB
CRGB leds[NUM_LEDS];

#define UPDATES_PER_SECOND 100
CRGBPalette16 currentPalette;
TBlendType currentBlending;

extern CRGBPalette16 myRedWhiteBluePalette;
extern const TProgmemPalette16 myRedWhiteBluePalette_p PROGMEM;



WiFiClient client;

Adafruit_MQTT_Client mqtt(&client, AIO_SERVER, AIO_SERVERPORT, AIO_USERNAME, AIO_KEY);

Adafruit_MQTT_Subscribe color = Adafruit_MQTT_Subscribe(&mqtt, AIO_USERNAME "/feeds/iot");
Adafruit_MQTT_Subscribe intensity = Adafruit_MQTT_Subscribe(&mqtt, AIO_USERNAME "/feeds/iot-intensity");
Adafruit_MQTT_Subscribe auto_mode = Adafruit_MQTT_Subscribe(&mqtt, AIO_USERNAME "/feeds/automatic-mode");
int automatic_mode = 1;
int inten = 100;
String col;
void MQTT_connect();

String openWeatherMapApiKey = "";
String city = "Bucharest";
String countryCode = "RO";
unsigned long lastTime = 0;
unsigned long timerDelay = 10000;  //wait 3 mins for weather update
String jsonBuffer;
#include <HTTPClient.h>
#include <Arduino_JSON.h>

void setup() {
  Serial.begin(115200);

  delay(10);
  Serial.println(F("Adafruit MQTT demo"));
  // Connect to WiFi access point.
  Serial.println();
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(WLAN_SSID);

  WiFi.begin(WLAN_SSID, WLAN_PASS);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println();

  Serial.println("WiFi connected");
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());

  // Setup MQTT subscription for onoff feed.
  mqtt.subscribe(&color);
  mqtt.subscribe(&intensity);
  mqtt.subscribe(&auto_mode);

  FastLED.addLeds<LED_TYPE, LED_PIN, COLOR_ORDER>(leds, NUM_LEDS).setCorrection(TypicalLEDStrip);
  FastLED.setBrightness(BRIGHTNESS);

  currentPalette = RainbowColors_p;
  currentBlending = LINEARBLEND;

  for (int j =0; j<4;j++){   
     for (int i = 0; i < NUM_LEDS; i++)
      leds[i] = CRGB(255, 255, 255);
    FastLED.show();
    delay(200);
         for (int i = 0; i < NUM_LEDS; i++)
      leds[i] = CRGB(0, 0, 0);
    FastLED.show();
      delay(200);
   }
}


void change_color() {
  if (col == "blue") {
    for (int i = 0; i < NUM_LEDS; i++)
      leds[i] = CRGB(0, 0, inten);
    FastLED.show();
  }
  if (col == "red") {
    for (int i = 0; i < NUM_LEDS; i++)
      leds[i] = CRGB(inten, 0, 0);
    FastLED.show();
  }
  if (col == "green") {
    for (int i = 0; i < NUM_LEDS; i++)
      leds[i] = CRGB(0, inten, 0);
    FastLED.show();
  }
  if (col == "yellow") {
    for (int i = 0; i < NUM_LEDS; i++)
      leds[i] = CRGB(inten, inten, 0);
    FastLED.show();
  }
  if (col == "purple") {
    for (int i = 0; i < NUM_LEDS; i++)
      leds[i] = CRGB(inten, 0, inten);
    FastLED.show();
  }

  if (col == "white") {
    for (int i = 0; i < NUM_LEDS; i++)
      leds[i] = CRGB(inten, inten, inten);
    FastLED.show();
  }
}
void loop() {
  MQTT_connect();


  Adafruit_MQTT_Subscribe *subscription;
  subscription = mqtt.readSubscription(20000);
  if (subscription == &auto_mode) {
    Serial.print(F("Auto: "));
    Serial.println((char *)auto_mode.lastread);
    String a((char *)auto_mode.lastread);
    if (a == "ON") {
      automatic_mode = 1;
      Serial.println("on");
    } else {
      automatic_mode = 0;
      change_color();
    }
  }


  if (subscription == &color) {
    Serial.print(F("Color: "));
    Serial.println((char *)color.lastread);

    String col1((char *)color.lastread);
    col = col1;
    Serial.println(col);
    if (automatic_mode == 0)
      change_color();
  }
  if (subscription == &intensity) {
    Serial.print(F("Intensity: "));
    Serial.println((char *)intensity.lastread);
    String inten1((char *)intensity.lastread);
    inten = inten1.toInt();
    inten = map(inten, 0, 100, 0, 255);
    Serial.println(inten);
    if (automatic_mode == 0) {
      change_color();
    }
  }

  if (automatic_mode == 1) {
    if ((millis() - lastTime) > timerDelay) {
      // Check WiFi connection status
      if (WiFi.status() == WL_CONNECTED) {
        String serverPath = "http://api.openweathermap.org/data/2.5/weather?q=" + city + "," + countryCode + "&APPID=" + openWeatherMapApiKey;

        jsonBuffer = httpGETRequest(serverPath.c_str());
        Serial.println(jsonBuffer);
        JSONVar myObject = JSON.parse(jsonBuffer);

        // JSON.typeof(jsonVar) can be used to get the type of the var
        if (JSON.typeof(myObject) == "undefined") {
          Serial.println("Parsing input failed!");
          return;
        }

        Serial.print("Temperature: ");
        int temp = round(myObject["main"]["temp"]) - 273;
        Serial.println(temp);
        Serial.print("Weather: ");
        Serial.println(myObject["weather"][0]["main"]);
        String weather = (myObject["weather"][0]["main"]);
        if (temp < -10)
          temp = -10;
        if (temp > 40)
          temp = 40;
        int intens = map(temp, -10, 40, 0, 255);
        if (weather == "Clouds") {
          for (int i = 0; i < NUM_LEDS; i++)
            leds[i] = CRGB(intens / 2, intens / 2, intens / 2);
          Serial.println(intens);
          FastLED.show();
        }
        if (weather == "Clear") {
          for (int i = 0; i < NUM_LEDS; i++)
            leds[i] = CRGB(intens, intens, 0);
          Serial.println(intens);
          FastLED.show();
        }
        
        if (weather == "Rain") {
          for (int i = 0; i < NUM_LEDS; i++)
            leds[i] = CRGB(0, 0, intens);
          Serial.println(intens);
          FastLED.show();
        }
      } else {
        Serial.println("WiFi Disconnected");
      }
      lastTime = millis();
    }
  }
}



void MQTT_connect() {
  int8_t ret;

  // Stop if already connected.
  if (mqtt.connected()) {
    return;
  }

  Serial.print("Connecting to MQTT... ");

  while ((ret = mqtt.connect()) != 0) {  // connect will return 0 for connected
    Serial.println(mqtt.connectErrorString(ret));
    Serial.println("Retrying MQTT connection in 5 seconds...");
    mqtt.disconnect();
    delay(5000);  // wait 5 seconds
  }
}











String httpGETRequest(const char *serverName) {
  WiFiClient client;
  HTTPClient http;

  // Your Domain name with URL path or IP address with path
  http.begin(client, serverName);

  // Send HTTP POST request
  int httpResponseCode = http.GET();

  String payload = "{}";

  if (httpResponseCode > 0) {
    Serial.print("HTTP Response code: ");
    Serial.println(httpResponseCode);
    payload = http.getString();
  } else {
    Serial.print("Error code: ");
    Serial.println(httpResponseCode);
  }
  // Free resources
  http.end();

  return payload;
}
